/* Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/* Turn LEDs in order from one edge on an Addressable RGB LED Strip from Pololu.
 *
 * To use this, you will need to plug an Addressable RGB LED strip from Pololu into pin 2.
 * Pin 3 and 4 are connected to pushbuttons to the ground, pin 3 is used as on/off, pin 4 switches color pallet 
 * After uploading the sketch, you should see a leds turning on and off in sequence.
 */
#include "button.h"
/*
 * Porting to AtTiny85 16.5MHz of the library from https://github.com/pololu/pololu-led-strip-arduino
 * See POLOLU_LICENSE.txt
 */
#include "PololuLedStrip2.h"

// define the pins that we are going to connect the push button to
#define BUTTON 0
#define PHOTO_CELL   A1 // pin 2 but analog channel 1
// Create a ledStrip object and specify the pin it will use.
PololuLedStrip<1> ledStrip; // using pin 1

// Create a buffer for holding the colors (3 bytes per color).
#define LED_COUNT 68
rgb_color colors[LED_COUNT];

// define the color pallet we are going to use in the application
// notice the first [] is left empty intentionally, the number of colors is identified at compile time
static uint8_t pallet [][3] = {
  {  0,  0,  0},
  { 64,  0, 32},
  {128,  0, 62},
  {255,  0,128},
  {  0, 64,255},
  {  0,128,255},
  {  0,255,255},
  {255,255,255},
};

#define MODE_OFF 0
#define MODE_DAY 4

// define parameters that will maintain value between function invocations
static uint8_t mode = 0;
static uint8_t oldMode = 1;
static uint8_t changeStart = 0;
static int8_t changeDelta = 0;
static uint16_t changeTs = 0;
static uint8_t powerPressedPrev = 0;

void PowerCB(uint16_t pressDuration)
{
  if(pressDuration > 30) // when power button is pressed 
  {
    changeTs = millis();
    if(mode == 0) // LEDs are currently off
    {
      int pc = analogRead(PHOTO_CELL);
      if(pc < 512) // dark room
      {
        if(oldMode < MODE_DAY)
        {
           mode = oldMode;
        }
        else
        {
          mode = 1;
        }
      }
      else // bright room
      {
          if(oldMode < MODE_DAY)
        {
           mode = MODE_DAY;
        }
        else
        {
           mode = oldMode;
        }
      }
      changeDelta = 1; // direction of propagation - from start to end
      changeStart = 0; // start index
    }
    else // LEDs are on -> turn them off from the tip to the handle
    {
      oldMode = mode; // save current color
      mode = 0; // move to off 
      changeDelta = -1;
      changeStart = LED_COUNT -1;
    }    
  }
}

void ModeCB(uint16_t pressDuration)
{
  if(mode != 0)
  {
    changeTs = millis();
    //changeStart = LED_COUNT -1;
    //changeDelta = -1;
    changeStart = 0;
    changeDelta = 1;
    mode++;
    if(mode == sizeof(pallet)/ sizeof(pallet[0]))
    {
      mode = 1; // skip mode 0 since it is off
    }
  }
  else // light is off, turn it on first
  {
    PowerCB(pressDuration);
  }
}


Button buttonPower(BUTTON, PowerCB, ModeCB, 500);

void setup()
{
    pinMode(BUTTON, INPUT);
}

void loop()
{
  uint16_t now = millis();
  buttonPower.Update();

  if(now - changeTs >= 2 && changeDelta != 0)
  {
    changeTs = now;
    if(changeDelta < 0) // turn LEDs off
    {
      if(changeStart == 0)
      {
        if(mode != 0)
        {
          changeDelta = 1;
        }
        else
        {
          changeDelta = 0;
        }
      }
      colors[changeStart].red = 0;
      colors[changeStart].green = 0;
      colors[changeStart].blue = 0;
      if(changeStart != 0)
      {
        changeStart += changeDelta;
      }
    }
    else // turn LEDs on
    {
      colors[changeStart].red  = pallet[mode][0];
      colors[changeStart].green = pallet[mode][1];
      colors[changeStart].blue = pallet[mode][2];
      changeStart += changeDelta;
      if(changeStart == LED_COUNT)
      {
        changeDelta = 0;
      }
    }
  }

  // Write the colors to the LED strip.
  ledStrip.write(colors, LED_COUNT);
}
